#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (C) 2020  David Arroyo Menéndez (davidam@gmail.com)
# This file is part of Damegender.

# Author: David Arroyo Menéndez <davidam@gmail.com>
# Author: Luz Galvis <newdaycodecontacto@gmail.com>
# Maintainer: David Arroyo Menéndez <davidam@gmail.com>

# This file is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.

# This file is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with DameGender; see the file GPL.txt.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301 USA,


class DameWikidata():

    def dicc_countries(self):
        # dictionary about wikidata codes and iso codes
        dicc = {"ad": "Q228",
                "ae": "Q878",
                "af": "Q889",
                "ag": "Q781",
                "al": "Q222",
                "ao": "Q916",
                "am": "Q399",
                "ar": "Q414",
                "at": "Q40",
                "au": "Q408",
                "az": "Q227",
                "ba": "Q188736",
                "bb": "Q244",
                "bd": "Q902",
                "be": "Q31",
                "bf": "Q965",
                "bg": "Q219",
                "bh": "Q398",
                "bi": "Q967",
                "bj": "Q962",
                "bn": "Q921",
                "bo": "Q750",
                "br": "Q155",
                "bs": "Q778",
                "bt": "Q917",
                "bw": "Q963",
                "by": "Q184",
                "bz": "Q242",
                "ca": "Q16",
                "cd": "Q974",
                "cf": "Q929",
                "cg": "Q971",
                "ch": "Q39",
                "ci": "Q1008",
                "cl": "Q298",
                "cm": "Q1009",
                "cn": "Q148",
                "co": "Q739",
                "cr": "Q800",
                "cu": "Q241",
                "cv": "Q1011",
                "cy": "Q229",
                "cz": "Q213",
                "de": "Q183",
                "dj": "Q977",
                "dk": "Q35",
                "dm": "Q784",
                "do": "Q786",
                "dz": "Q262",
                "ec": "Q736",
                "ee": "Q191",
                "eg": "Q79",
                "er": "Q986",
                "es": "Q29",
                "et": "Q115",
                "fi": "Q33",
                "fj": "Q712",
                "fm": "Q3359409",
                "fr": "Q142",
                "ga": "Q1000",
                "gb": "Q145",
                "ge": "Q1428",
                "gh": "Q117",
                "gm": "Q1005",
                "gn": "Q1006",
                "gq": "Q983",
                "gr": "Q41",
                "gt": "Q774",
                "gw": "Q1007",
                "gy": "Q734",
                "hn": "Q783",
                "hr": "Q224",
                "ht": "Q790",
                "hu": "Q28",
                "id": "Q252",
                "ie": "Q27",
                "il": "Q801",
                "in": "Q668",
                "iq": "Q796",
                "ir": "Q794",
                "is": "Q189",
                "it": "Q38",
                "jm": "Q766",
                "jo": "Q810",
                "jp": "Q17",
                "ke": "Q114",
                "kg": "Q813",
                "kh": "Q424",
                "ki": "Q710",
                "km": "Q970",
                "kn": "Q763",
                "kp": "Q423",
                "kr": "Q9176",
                "kw": "Q817",
                "kz": "Q232",
                "la": "Q819",
                "lb": "Q822",
                "lc": "Q760",
                "li": "Q347",
                "lk": "Q854",
                "lr": "Q1014",
                "ls": "Q1013",
                "lt": "Q37",
                "lu": "Q1842",
                "lv": "Q211",
                "ly": "Q1016",
                "ma": "Q1028",
                "mc": "Q235",
                "md": "Q217",
                "me": "Q236",
                "mg": "Q1019",
                "mh": "Q709",
                "mk": "Q221",
                "ml": "Q912",
                "mm": "Q836",
                "mn": "Q711",
                "mr": "Q1025",
                "mt": "Q233",
                "mu": "Q1027",
                "mv": "Q826",
                "mw": "Q1020",
                "mx": "Q96",
                "my": "Q833",
                "mz": "Q1029",
                "na": "Q1030",
                "ne": "Q1032",
                "ng": "Q1033",
                "ni": "Q811",
                "nl": "Q55",
                "no": "Q20",
                "np": "Q837",
                "nr": "Q697",
                "nz": "Q664",
                "om": "Q842",
                "pa": "Q804",
                "pe": "Q419",
                "pg": "Q691",
                "ph": "Q928",
                "pk": "Q843",
                "pl": "Q36",
                "pt": "Q45",
                "pw": "Q695",
                "py": "Q733",
                "qa": "Q846",
                "ro": "Q218",
                "rs": "Q403",
                "ru": "Q159",
                "rw": "Q1037",
                "sa": "Q851",
                "sb": "Q685",
                "sc": "Q1042",
                "sd": "Q1049",
                "se": "Q34",
                "sg": "Q334",
                "si": "Q215",
                "sk": "Q214",
                "sl": "Q1044",
                "sm": "Q238",
                "sn": "Q1041",
                "so": "Q1045",
                "sr": "Q730",
                "ss": "Q958",
                "st": "Q1039",
                "sv": "Q792",
                "sy": "Q858",
                "sz": "Q1050",
                "td": "Q657",
                "tg": "Q945",
                "th": "Q869",
                "tj": "Q863",
                "tl": "Q574",
                "tm": "Q874",
                "tn": "Q948",
                "to": "Q678",
                "tr": "Q43",
                "tt": "Q754",
                "tv": "Q672",
                "tw": "Q22502",
                "tz": "Q924",
                "ua": "Q212",
                "ug": "Q1036",
                "us": "Q35657",
                "uy": "Q77",
                "uz": "Q265",
                "va": "Q237",
                "vc": "Q757",
                "ve": "Q717",
                "vn": "Q881",
                "vu": "Q686",
                "ws": "Q683",
                "ye": "Q805",
                "za": "Q550374",
                "zm": "Q953",
                "zw": "Q954"
                }
        return dicc
